@@ -4,14 +4,14 @@
 # Licensed under the terms of the MIT License
 # (see spyderlib/__init__.py for details)
 
-"""Workspace widget"""
+"""Workspace Plugin"""
 
 # pylint: disable-msg=C0103
 # pylint: disable-msg=R0903
 # pylint: disable-msg=R0911
 # pylint: disable-msg=R0201
 
-from PyQt4.QtGui import QFileDialog, QMessageBox, QFontDialog, QMenu
+from PyQt4.QtGui import QMessageBox, QFontDialog, QMenu
 from PyQt4.QtCore import SIGNAL
 
 import os, sys
@@ -23,12 +23,11 @@ STDOUT = sys.stdout
 # Local imports
 from spyderlib.config import (CONF, get_conf_path, str2type, get_icon,
                               get_font, set_font)
-from spyderlib.utils.qthelpers import create_action, add_actions, translate
-from spyderlib.utils.iofuncs import (save_dictionary, load_dictionary,
-                                     load_array, load_image, load_dicom,
-                                     load_matlab, save_matlab)
+from spyderlib.utils import fix_reference_name
+from spyderlib.utils.qthelpers import create_action, add_actions
+from spyderlib.utils.iofuncs import iofunctions
 from spyderlib.widgets.dicteditor import DictEditorTableView, globalsfilter
-from spyderlib.plugins import PluginMixin
+from spyderlib.plugins import SpyderPluginMixin
 
 
 FILTERS = tuple(str2type(CONF.get('workspace', 'filters')))
@@ -47,7 +46,7 @@ def wsfilter(input_dict, itermax=ITERMAX, filters=FILTERS):
                          excluded_names=excluded_names)
 
 
-class Workspace(DictEditorTableView, PluginMixin):
+class Workspace(DictEditorTableView, SpyderPluginMixin):
     """
     Workspace widget (namespace explorer)
     """
@@ -65,59 +64,14 @@ class Workspace(DictEditorTableView, PluginMixin):
         DictEditorTableView.__init__(self, parent, None, names=True,
                                      truncate=truncate, inplace=inplace,
                                      minmax=minmax, collvalue=collvalue)
-        PluginMixin.__init__(self, parent)
+        SpyderPluginMixin.__init__(self, parent)
         
-        self.setup_io()
         self.load_temp_namespace()
         
         self.setFont(get_font(self.ID))
 
-    def setup_io(self):
-        """Setup I/O functions and filters"""
-        iofuncs = (
-                   ('.spydata', translate('Workspace', "Spyder data files"),
-                                load_dictionary, save_dictionary),
-                   ('.npy',     translate('Workspace', "NumPy arrays"),
-                                load_array, None),
-                   ('.mat',     translate('Workspace', "Matlab files"),
-                                load_matlab, save_matlab),
-                   ('.csv',     translate('Workspace', "CSV text files"),
-                                'import_wizard', None),
-                   ('.txt',     translate('Workspace', "Text files"),
-                                'import_wizard', None),
-                   ('.jpg',     translate('Workspace', "JPEG images"),
-                                load_image, None),
-                   ('.png',     translate('Workspace', "PNG images"),
-                                load_image, None),
-                   ('.gif',     translate('Workspace', "GIF images"),
-                                load_image, None),
-                   ('.tif',     translate('Workspace', "TIFF images"),
-                                load_image, None),
-                   ('.dcm',     translate('Workspace', "DICOM images"),
-                                load_dicom, None),
-                   )
-        load_funcs = {}
-        save_funcs = {}
-        load_filters = []
-        save_filters = []
-        load_ext = []
-        for ext, name, loadfunc, savefunc in iofuncs:
-            filter_str = unicode(name + " (*%s)" % ext)
-            if loadfunc is not None:
-                load_filters.append(filter_str)
-                load_funcs[ext] = loadfunc
-                load_ext.append(ext)
-            if savefunc is not None:
-                save_filters.append(filter_str)
-                save_funcs[ext] = savefunc
-        load_filters.insert(0, unicode(self.tr("Supported files")+" (*"+\
-                                       " *".join(load_ext)+")"))
-        self.load_filters = "\n".join(load_filters)
-        self.save_filters = "\n".join(save_filters)
-        self.load_funcs = load_funcs
-        self.save_funcs = save_funcs
-        
-    def get_widget_title(self):
+    #------ SpyderPluginWidget API ---------------------------------------------    
+    def get_plugin_title(self):
         """Return widget title"""
         return self.tr('Workspace')
     
@@ -128,44 +82,7 @@ class Workspace(DictEditorTableView, PluginMixin):
         """
         return self
         
-    def set_interpreter(self, interpreter):
-        """Bind to interpreter"""
-        self.interpreter = interpreter
-        self.refresh(force=True)
-        
-    def get_namespace(self, itermax=ITERMAX):
-        """Return filtered namespace"""
-        return wsfilter(self.namespace, itermax=itermax)
-    
-    def __clear_namespace(self):
-        """Clear namespace"""
-        keys = self.get_namespace().keys()
-        for key in keys:
-            self.namespace.pop(key)
-        self.refresh(force=True)
-    
-    def clear(self):
-        """Ask to clear workspace"""
-        answer = QMessageBox.question(self, self.tr("Clear workspace"),
-                    self.tr("Do you want to clear all data from workspace?"),
-                    QMessageBox.Yes | QMessageBox.No)
-        if answer == QMessageBox.Yes:
-            self.__clear_namespace()
-
-    def refresh(self, force=False):
-        """Refresh widget"""
-        if CONF.get(self.ID, 'autorefresh') or force:
-            self.refresh_editor()
-        
-    def refresh_editor(self):
-        """Refresh DictEditor"""
-        if self.interpreter is not None:
-            self.namespace = self.interpreter.namespace
-        self.set_filter( wsfilter )
-        self.set_data( self.namespace )
-        self.adjust_columns()
-        
-    def set_actions(self):
+    def get_plugin_actions(self):
         """Setup actions"""
         import_action = create_action(self, self.tr("Import data..."), None,
             'ws_open.png', self.tr("Import data to workspace"),
@@ -232,31 +149,13 @@ class Workspace(DictEditorTableView, PluginMixin):
                         option_menu)
         toolbar_actions = (refresh_action, import_action, save_as_action)
         return (menu_actions, toolbar_actions)
+
+    def refresh_plugin(self, force=False):
+        """Refresh widget"""
+        if CONF.get(self.ID, 'autorefresh') or force:
+            self.refresh_editor()
         
-    def change_font1(self):
-        """Change font"""
-        self.__change_font('dicteditor_header')
-        
-    def change_font2(self):
-        """Change font"""
-        self.__change_font('dicteditor')
-    
-    def __change_font(self, section):
-        font, valid = QFontDialog.getFont(get_font(section), self,
-                                          self.tr("Select a new font"))
-        if valid:
-            set_font(font, section)
-    
-    def toggle_autorefresh(self, checked):
-        """Toggle autorefresh mode"""
-        CONF.set(self.ID, 'autorefresh', checked)
-        self.refresh()
-        
-    def toggle_autosave(self, checked):
-        """Toggle autosave mode"""
-        CONF.set(self.ID, 'autosave', checked)
-        
-    def closing(self, cancelable=False):
+    def closing_plugin(self, cancelable=False):
         """Perform actions before parent main window is closed"""
         if CONF.get(self.ID, 'autosave'):
             # Saving workspace
@@ -278,7 +177,7 @@ class Workspace(DictEditorTableView, PluginMixin):
                 buttons = QMessageBox.Yes | QMessageBox.No
                 if cancelable:
                     buttons = buttons | QMessageBox.Cancel
-                answer = QMessageBox.question(self, self.get_widget_title(),
+                answer = QMessageBox.question(self, self.get_plugin_title(),
                    self.tr("Workspace is currently keeping reference "
                            "to %1 object%2.\n\nDo you want to save %3?") \
                    .arg(srefnb).arg(s_or_not).arg(it_or_them), buttons)
@@ -291,6 +190,62 @@ class Workspace(DictEditorTableView, PluginMixin):
                     # Removing last saved workspace
                     os.remove(self.TEMPFILE_PATH)
         return True
+        
+    #------ Public API ---------------------------------------------------------
+    def set_interpreter(self, interpreter):
+        """Bind to interpreter"""
+        self.interpreter = interpreter
+        self.refresh_plugin(force=True)
+        
+    def get_namespace(self, itermax=ITERMAX):
+        """Return filtered namespace"""
+        return wsfilter(self.namespace, itermax=itermax)
+    
+    def __clear_namespace(self):
+        """Clear namespace"""
+        keys = self.get_namespace().keys()
+        for key in keys:
+            self.namespace.pop(key)
+        self.refresh_plugin(force=True)
+    
+    def clear(self):
+        """Ask to clear workspace"""
+        answer = QMessageBox.question(self, self.tr("Clear workspace"),
+                    self.tr("Do you want to clear all data from workspace?"),
+                    QMessageBox.Yes | QMessageBox.No)
+        if answer == QMessageBox.Yes:
+            self.__clear_namespace()
+        
+    def refresh_editor(self):
+        """Refresh DictEditor"""
+        if self.interpreter is not None:
+            self.namespace = self.interpreter.namespace
+        self.set_filter( wsfilter )
+        self.set_data( self.namespace )
+        self.adjust_columns()
+        
+    def change_font1(self):
+        """Change font"""
+        self.__change_font('dicteditor_header')
+        
+    def change_font2(self):
+        """Change font"""
+        self.__change_font('dicteditor')
+    
+    def __change_font(self, section):
+        font, valid = QFontDialog.getFont(get_font(section), self,
+                                          self.tr("Select a new font"))
+        if valid:
+            set_font(font, section)
+    
+    def toggle_autorefresh(self, checked):
+        """Toggle autorefresh mode"""
+        CONF.set(self.ID, 'autorefresh', checked)
+        self.refresh_plugin()
+        
+    def toggle_autosave(self, checked):
+        """Toggle autosave mode"""
+        CONF.set(self.ID, 'autosave', checked)
     
     def load_temp_namespace(self):
         """Attempt to load last session namespace"""
@@ -299,7 +254,7 @@ class Workspace(DictEditorTableView, PluginMixin):
             self.import_data(self.filename)
         else:
             self.namespace = None
-            self.refresh(force=True)
+            self.refresh_plugin(force=True)
 
     def import_data(self, filename=None):
         """
@@ -310,17 +265,16 @@ class Workspace(DictEditorTableView, PluginMixin):
         if filename is None:
             self.emit(SIGNAL('redirect_stdio(bool)'), False)
             basedir = osp.dirname(self.filename)
-            filename = QFileDialog.getOpenFileName(self,
-                          title, basedir, self.load_filters)
+            filename = iofunctions.get_open_filename(self, basedir, title)
             self.emit(SIGNAL('redirect_stdio(bool)'), True)
             if filename:
                 filename = unicode(filename)
             else:
                 return
         self.filename = unicode(filename)
-        ext = osp.splitext(self.filename)[1]
+        ext = osp.splitext(self.filename)[1].lower()
         
-        if ext not in self.load_funcs:
+        if ext not in iofunctions.load_funcs:
             buttons = QMessageBox.Yes | QMessageBox.Cancel
             answer = QMessageBox.question(self, title,
                        self.tr("<b>Unsupported file type '%1'</b><br><br>"
@@ -331,7 +285,7 @@ class Workspace(DictEditorTableView, PluginMixin):
             else:
                 load_func = 'import_wizard'
         else:
-            load_func = self.load_funcs[ext]
+            load_func = iofunctions.load_funcs[ext]
             
         if isinstance(load_func, basestring): # 'import_wizard' (self.setup_io)
             # Import data with import wizard
@@ -345,7 +299,13 @@ class Workspace(DictEditorTableView, PluginMixin):
         else:
             self.starting_long_process(self.tr("Loading data..."))
             namespace, error_message = load_func(self.filename)
-            self.ending_long_process()           
+            if self.namespace is not None:
+                for key in namespace.keys():
+                    new_key = fix_reference_name(key,
+                                 blacklist=self.interpreter.namespace.keys())
+                    if new_key != key:
+                        namespace[new_key] = namespace.pop(key)
+            self.ending_long_process()
             if error_message is None:
                 if self.namespace is None:
                     self.namespace = namespace
@@ -357,14 +317,13 @@ class Workspace(DictEditorTableView, PluginMixin):
                                  self.tr("<b>Unable to load '%1'</b>"
                                          "<br><br>Error message:<br>%2") \
                                          .arg(self.filename).arg(error_message))
-        self.refresh(force=True)
+        self.refresh_plugin(force=True)
 
     def save_as(self):
         """Save current workspace as"""
         self.emit(SIGNAL('redirect_stdio(bool)'), False)
-        filename = QFileDialog.getSaveFileName(self,
-                      self.tr("Save workspace"), self.filename,
-                      self.save_filters)
+        filename = iofunctions.get_save_filename(self, self.filename,
+                                                  self.tr("Save workspace"))
         self.emit(SIGNAL('redirect_stdio(bool)'), True)
         if filename:
             self.filename = unicode(filename)
@@ -376,16 +335,9 @@ class Workspace(DictEditorTableView, PluginMixin):
         """Save workspace"""
         title = self.tr("Save workspace")
         
-        ext = osp.splitext(filename)[1]
-        if ext not in self.save_funcs:
-            QMessageBox.critical(self, title,
-                                 self.tr("<b>Unsupported file type '%1'</b>") \
-                                         .arg(ext))
-            return
-        
         self.starting_long_process(self.tr("Saving workspace..."))
         namespace = self.get_namespace(itermax=-1).copy()
-        error_message = self.save_funcs[ext](namespace, filename)
+        error_message = iofunctions.save(namespace, filename)
         self.ending_long_process()
         
         if error_message is not None:
@@ -394,23 +346,23 @@ class Workspace(DictEditorTableView, PluginMixin):
                                     "<br><br>Error message:<br>%1") \
                             .arg(error_message))
             
-        self.refresh(force=True)
+        self.refresh_plugin(force=True)
         return True
 
     def toggle_exclude_private(self, checked):
         """Toggle exclude private references"""
         CONF.set(self.ID, 'exclude_private', checked)
-        self.refresh(force=True)
+        self.refresh_plugin(force=True)
         
     def toggle_exclude_upper(self, checked):
         """Toggle exclude upper-case references"""
         CONF.set(self.ID, 'exclude_upper', checked)
-        self.refresh(force=True)
+        self.refresh_plugin(force=True)
 
     def toggle_exclude_unsupported(self, checked):
         """Toggle exclude unsupported datatypes"""
         CONF.set(self.ID, 'exclude_unsupported', checked)
-        self.refresh(force=True)
+        self.refresh_plugin(force=True)
 
     #----Focus
     def focusInEvent(self, event):
