@@ -54,14 +54,17 @@ def getdoc(obj):
 def getsource(obj):
     """Wrapper around inspect.getsource"""
     try:
-        src = encoding.to_unicode( inspect.getsource(obj) )
-    except TypeError:
-        if hasattr(obj, '__class__'):
-            src = encoding.to_unicode( inspect.getsource(obj.__class__) )
-        else:
-            # Bindings like VTK or ITK require this case
-            src = getdoc(obj)
-    return src
+        try:
+            src = encoding.to_unicode( inspect.getsource(obj) )
+        except TypeError:
+            if hasattr(obj, '__class__'):
+                src = encoding.to_unicode( inspect.getsource(obj.__class__) )
+            else:
+                # Bindings like VTK or ITK require this case
+                src = getdoc(obj)
+        return src
+    except (TypeError, IOError):
+        return
 
 def getargfromdoc(obj):
     """Get arguments from object doc"""
@@ -119,7 +122,8 @@ def getargtxt(obj, one_arg_per_line=True):
 
 
 def isdefined(obj, force_import=False, namespace=None):
-    """Return True if object is defined"""
+    """Return True if object is defined in namespace
+    If namespace is None --> namespace = locals()"""
     if namespace is None:
         namespace = locals()
     attr_list = obj.split('.')
@@ -127,12 +131,12 @@ def isdefined(obj, force_import=False, namespace=None):
     if len(base) == 0:
         return False
     import __builtin__
-    if base not in locals() and base not in globals() \
-       and base not in __builtin__.__dict__ and base not in namespace:
+    if base not in __builtin__.__dict__ and base not in namespace:
         if force_import:
             try:
                 module = __import__(base, globals(), namespace)
-                globals()[base] = module
+                if base not in globals():
+                    globals()[base] = module
                 namespace[base] = module
             except ImportError:
                 return False
