@@ -7,15 +7,16 @@
 """Qt utilities"""
 
 import os.path as osp
-import os, webbrowser, imp
+import os, webbrowser
 
 from PyQt4.QtGui import (QAction, QStyle, QWidget, QIcon, QApplication,
                          QVBoxLayout, QHBoxLayout, QLineEdit, QLabel,
                          QKeySequence, QToolButton, QKeyEvent, QMenu)
-from PyQt4.QtCore import SIGNAL, QVariant, QObject, Qt
+from PyQt4.QtCore import (SIGNAL, QVariant, QObject, Qt, QLocale, QTranslator,
+                          QLibraryInfo)
 
 # Local import
-from spyderlib.config import get_icon
+from spyderlib.config import get_icon, DATA_PATH
 from spyderlib.utils import programs
 
 # Note: How to redirect a signal from widget *a* to widget *b* ?
@@ -28,6 +29,32 @@ from spyderlib.utils import programs
 #    self.connect(self.listwidget, SIGNAL('option_changed'),
 #                 lambda *args: self.emit(SIGNAL('option_changed'), *args))
 
+TRANSLATORS = []
+
+def qapplication(translate=True):
+    """
+    Return QApplication instance
+    Creates it if it doesn't already exist
+    """
+    if QApplication.startingUp():
+        app = QApplication([])
+        if translate:
+            locale = QLocale.system().name()
+            # Qt-specific translator
+            qt_translator = QTranslator()
+            TRANSLATORS.append(qt_translator) # Keep reference alive
+            paths = QLibraryInfo.location(QLibraryInfo.TranslationsPath)
+            if qt_translator.load("qt_"+locale, paths):
+                app.installTranslator(qt_translator)
+            # Spyder-specific translator
+            app_translator = QTranslator()
+            TRANSLATORS.append(app_translator) # Keep reference alive
+            if app_translator.load("spyder_"+locale, DATA_PATH):
+                app.installTranslator(app_translator)
+    else:
+        app = QApplication.instance()
+    return app
+
 def translate(context, string):
     """Translation"""
     return QApplication.translate(context, string)
@@ -37,12 +64,24 @@ def keybinding(attr):
     ks = getattr(QKeySequence, attr)
     return QKeySequence.keyBindings(ks)[0].toString()
 
-def mimedata2url(source):
-    """Extract url list from MIME data"""
+def mimedata2url(source, extlist=None):
+    """
+    Extract url list from MIME data
+    extlist: for example ('.py', '.pyw')
+    """
     if source.hasUrls():
-        paths = [unicode(url.toString()) for url in source.urls()]
-        return [path[8:] for path in paths if path.startswith(r"file://") \
-                and (path.endswith(".py") or path.endswith(".pyw"))]
+        pathlist = []
+        for url in source.urls():
+            path = unicode(url.toString())
+            if path.startswith(r"file://"):
+                if os.name == 'nt':
+                    path = path[8:]
+                else:
+                    path = path[7:]
+            if osp.exists(path):
+                if extlist is None or osp.splitext(path)[1] in extlist:
+                    pathlist.append(path)
+        return pathlist
 
 def keyevent2tuple(event):
     """Convert QKeyEvent instance into a tuple"""
@@ -66,22 +105,28 @@ def restore_keyevent(event):
     return event, text, key, ctrl, shift
 
 def create_toolbutton(parent, icon=None, text=None,
-                      triggered=None, tip=None, toggled=None):
+                      triggered=None, tip=None, toggled=None, shortcut=None,
+                      autoraise=True, text_beside_icon=True):
     """Create a QToolButton"""
     button = QToolButton(parent)
     if text is not None:
         button.setText(text)
     if icon is not None:
+        if isinstance(icon, (str, unicode)):
+            icon = get_icon(icon)
         button.setIcon(icon)
     if text is not None or tip is not None:
         button.setToolTip(text if tip is None else tip)
-    button.setToolButtonStyle(Qt.ToolButtonTextBesideIcon)
-    button.setAutoRaise(True)
+    if text_beside_icon:
+        button.setToolButtonStyle(Qt.ToolButtonTextBesideIcon)
+    button.setAutoRaise(autoraise)
     if triggered is not None:
         QObject.connect(button, SIGNAL('clicked()'), triggered)
     if toggled is not None:
         QObject.connect(button, SIGNAL("toggled(bool)"), toggled)
         button.setCheckable(True)
+    if shortcut is not None:
+        button.setShortcut(shortcut)
     return button
 
 def toggle_actions(actions, enable):
@@ -93,7 +138,7 @@ def toggle_actions(actions, enable):
 
 def create_action(parent, text, shortcut=None, icon=None, tip=None,
                   toggled=None, triggered=None, data=None,
-                  window_context=True):
+                  context=Qt.WindowShortcut):
     """Create a QAction"""
     action = QAction(text, parent)
     if triggered is not None:
@@ -112,13 +157,10 @@ def create_action(parent, text, shortcut=None, icon=None, tip=None,
         action.setStatusTip(tip)
     if data is not None:
         action.setData(QVariant(data))
-    if window_context:
-        action.setShortcutContext(Qt.WindowShortcut)
-    else:
-        #TODO: Hard-code all shortcuts and choose window_context=False
-        # (this will avoid calling shortcuts from another dockwidget
-        #  since the context thing doesn't work quite well with these)
-        action.setShortcutContext(Qt.WidgetShortcut)
+    #TODO: Hard-code all shortcuts and choose context=Qt.WidgetShortcut
+    # (this will avoid calling shortcuts from another dockwidget
+    #  since the context thing doesn't work quite well with these widgets)
+    action.setShortcutContext(context)
     return action
 
 def add_actions(target, actions, insert_before=None):
@@ -147,37 +189,45 @@ def add_actions(target, actions, insert_before=None):
                 target.insertAction(insert_before, action)
         previous_action = action
 
-def add_bookmark(parent, menu, url, title, icon=None, shortcut=None):
-    """Add bookmark to a menu"""
+
+def get_item_user_text(item):
+    """Get QTreeWidgetItem user role string"""
+    return unicode(item.data(0, Qt.UserRole).toString())
+
+def set_item_user_text(item, text):
+    """Set QTreeWidgetItem user role string"""
+    item.setData(0, Qt.UserRole, QVariant(text))
+
+
+def create_bookmark_action(parent, url, title, icon=None, shortcut=None):
+    """Create bookmark action"""
     if icon is None:
         icon = get_icon('browser.png')
     if os.name == 'nt':
         callback = os.startfile
     else:
         callback = webbrowser.open
-    act = create_action( parent, title, shortcut=shortcut, icon=icon,
-                         triggered=lambda u=url: callback(u) )
-    menu.addAction(act)
+    return create_action( parent, title, shortcut=shortcut, icon=icon,
+                          triggered=lambda u=url: callback(u) )
 
-def add_module_dependent_bookmarks(parent, menu, bookmarks):
+def create_module_bookmark_actions(parent, bookmarks):
     """
-    Add bookmarks to a menu depending on module installation:
+    Create bookmark actions depending on module installation:
     bookmarks = ((module_name, url, title), ...)
     """
+    actions = []
     for key, url, title, icon in bookmarks:
-        try:
-            imp.find_module(key)
-            add_bookmark(parent, menu, url, title, get_icon(icon))
-        except ImportError:
-            pass
+        if programs.is_module_installed(key):
+            act = create_bookmark_action(parent, url, title, get_icon(icon))
+            actions.append(act)
+    return actions
         
 def create_program_action(parent, text, icon, name, nt_name=None):
     """Create action to run a program"""
     if os.name == 'nt':
-        if nt_name is None:
-            name += ".exe"
-        else:
+        if nt_name is not None:
             name = nt_name
+        name = programs.get_nt_program_name(name)
     if isinstance(icon, basestring):
         icon = get_icon(icon)
     if programs.is_program_installed(name):
@@ -243,10 +293,10 @@ def show_std_icons():
     """
     Show all standard Icons
     """
-    import sys
-    app = QApplication(sys.argv)
+    app = qapplication()
     dialog = ShowStdIcons(None)
     dialog.show()
+    import sys
     sys.exit(app.exec_())
 
 if __name__ == "__main__":
