@@ -26,7 +26,9 @@ from PyQt4.QtCore import (Qt, QVariant, QModelIndex, QAbstractTableModel,
 
 # Local import
 from spyderlib.config import get_icon, get_font
-from spyderlib.utils.qthelpers import translate, add_actions, create_action
+from spyderlib.utils import fix_reference_name
+from spyderlib.utils.qthelpers import (translate, add_actions, create_action,
+                                       qapplication)
 from spyderlib.widgets.texteditor import TextEditor
 from spyderlib.widgets.importwizard import ImportWizard
 
@@ -42,6 +44,14 @@ except ImportError:
         """Fake ndarray"""
         pass
 
+#----PIL Images support
+try:
+    from PIL.Image import Image
+except:
+    class Image(FakeObject):
+        """Fake PIL Image"""
+        pass
+
 #----Misc.
 def address(obj):
     """Return object address as a string: '<classname @ address>'"""
@@ -70,6 +80,7 @@ COLORS = {
           tuple: Qt.lightGray,
           (str, unicode): Qt.darkRed,
           ndarray: Qt.green,
+          Image: Qt.darkGreen,
           datetime.date: Qt.darkYellow,
           }
 
@@ -104,6 +115,8 @@ def value_to_display(value, truncate=False,
             return 'Min: %r\nMax: %r' % (value.min(), value.max())
         except TypeError:
             pass
+    if isinstance(value, Image):
+        return '%s  Mode: %s' % (address(value), value.mode)
     if not isinstance(value, (str, unicode)):
         if isinstance(value, (list, tuple, dict, set)) and not collvalue:            
             value = address(value)
@@ -148,6 +161,8 @@ def get_size(item):
         return len(item)
     elif isinstance(item, ndarray):
         return item.shape
+    elif isinstance(item, Image):
+        return item.size
     else:
         return 1
 
@@ -158,6 +173,8 @@ def get_type(item):
            if not found else found[0]
     if isinstance(item, ndarray):
         text = item.dtype.name
+    if isinstance(item, Image):
+        text = "Image"
     return text[text.find('.')+1:]
 
 
@@ -419,6 +436,11 @@ class DictDelegate(QItemDelegate):
                     # (ArrayEditor does not make a copy of value)
                     self.set_value(index, value)
             return None
+        #---showing image
+        elif isinstance(value, Image) and ndarray is not FakeObject \
+             and Image is not FakeObject:
+            value.show()
+            return None
         #---editor = QDateTimeEdit
         elif isinstance(value, datetime.datetime) and not self.inplace:
             editor = QDateTimeEdit(value, parent)
@@ -501,6 +523,8 @@ class BaseTableView(QTableView):
     def __init__(self, parent):
         QTableView.__init__(self, parent)
         self.array_filename = None
+        self.menu = None
+        self.empty_ws_menu = None
         
     def setup_table(self):
         """Setup table"""
@@ -512,6 +536,17 @@ class BaseTableView(QTableView):
     
     def setup_menu(self, truncate, minmax, inplace, collvalue):
         """Setup context menu"""
+        self.empty_ws_menu = QMenu(self)
+        self.paste_action = create_action(self,
+                                      translate("DictEditor", "Paste"),
+                                      icon=get_icon('editpaste.png'),
+                                      triggered=self.paste)
+        self.empty_ws_menu.addAction(self.paste_action)
+        
+        self.copy_action = create_action(self,
+                                      translate("DictEditor", "Copy"),
+                                      icon=get_icon('editcopy.png'),
+                                      triggered=self.copy)                                      
         self.edit_action = create_action(self, 
                                       translate("DictEditor", "Edit"),
                                       icon=get_icon('edit.png'),
@@ -573,10 +608,10 @@ class BaseTableView(QTableView):
                                     triggered=self.duplicate_item)
         menu = QMenu(self)
         menu_actions = [self.edit_action, self.plot_action, self.imshow_action,
-                        self.save_array_action,
-                        self.insert_action, self.remove_action, None,
-                        self.rename_action,self.duplicate_action, None,
-                        self.truncate_action, self.inplace_action,
+                        self.save_array_action, self.insert_action,
+                        self.remove_action, self.copy_action, self.paste_action,
+                        None, self.rename_action,self.duplicate_action,
+                        None, self.truncate_action, self.inplace_action,
                         self.collvalue_action]
         if ndarray is not FakeObject:
             menu_actions.append(self.minmax_action)
@@ -594,12 +629,13 @@ class BaseTableView(QTableView):
     def refresh_plot_entries(self, index):
         if index.isValid():
             value = self.delegate.get_value(index)
+            is_list = isinstance(value, (tuple, list))
             is_array = isinstance(value, ndarray) and len(value) != 0
-            condition_plot = is_array and len(value.shape) <= 2
-            condition_imshow = condition_plot and min(value.shape) > 2
+            condition_plot = (is_array and len(value.shape) <= 2)
+            condition_imshow = condition_plot and value.ndim == 2
         else:
             is_array = condition_plot = condition_imshow = False
-        self.plot_action.setVisible(condition_plot)
+        self.plot_action.setVisible(condition_plot or is_list)
         self.imshow_action.setVisible(condition_imshow)
         self.save_array_action.setVisible(is_array)
         
@@ -638,6 +674,9 @@ class BaseTableView(QTableView):
         elif event.key() == Qt.Key_F2:
             self.rename_item()
             event.accept()
+        elif event == QKeySequence.Copy:
+            self.copy()
+            event.accept()
         elif event == QKeySequence.Paste:
             self.paste()
             event.accept()
@@ -689,11 +728,13 @@ class BaseTableView(QTableView):
         for index in indexes:
             if not index.isValid():
                 return
+        one = translate("DictEditor", "Do you want to remove selected item?")
+        more = translate("DictEditor",
+                         "Do you want to remove all selected items?")
         answer = QMessageBox.question(self,
-            translate("DictEditor", "Remove"),
-            translate("DictEditor", "Do you want to remove selected item%1?") \
-            .arg('s' if len(indexes)>1 else ''),
-            QMessageBox.Yes | QMessageBox.No)
+                                      translate("DictEditor", "Remove"),
+                                      one if len(indexes) == 1 else more,
+                                      QMessageBox.Yes | QMessageBox.No)
         if answer == QMessageBox.Yes:
             idx_rows = unsorted_unique(map(lambda idx: idx.row(), indexes))
             keys = [ self.model.keys[idx_row] for idx_row in idx_rows ]
@@ -759,8 +800,9 @@ class BaseTableView(QTableView):
             
     def __prepare_plot(self):
         try:
-            from matplotlib import rcParams
-            rcParams['backend'] = 'Qt4agg'
+            from spyderlib import mpl_patch
+            mpl_patch.configure(backend="Qt4Agg")
+            mpl_patch.apply()
             return True
         except ImportError:
             QMessageBox.warning(self, translate("DictEditor", "Import error"),
@@ -771,19 +813,37 @@ class BaseTableView(QTableView):
         """Plot item"""
         index = self.currentIndex()
         if self.__prepare_plot():
-            from pylab import plot, show
+            try:
+                import guiqwt.pyplot as plt
+            except ImportError:
+                import matplotlib.pyplot as plt
             data = self.delegate.get_value(index)
-            plot(data)
-            show()
+            try:
+                plt.figure()
+                plt.plot(data)
+                plt.show()
+            except ValueError, error:
+                QMessageBox.critical(self, translate("DictEditor", "Plot"),
+                    translate("DictEditor", "<b>Unable to plot data.</b>"
+                              "<br><br>Error message:<br>%1").arg(str(error)))
             
     def imshow_item(self):
         """Imshow item"""
         index = self.currentIndex()
         if self.__prepare_plot():
-            from pylab import imshow, show
+            try:
+                import guiqwt.pyplot as plt
+            except ImportError:
+                import matplotlib.pyplot as plt
             data = self.delegate.get_value(index)
-            imshow(data)
-            show()
+            try:
+                plt.figure()
+                plt.imshow(data)
+                plt.show()
+            except ValueError, error:
+                QMessageBox.critical(self, translate("DictEditor", "Plot"),
+                    translate("DictEditor", "<b>Unable to show image.</b>"
+                              "<br><br>Error message:<br>%1").arg(str(error)))
             
     def save_array(self):
         """Save array"""
@@ -804,6 +864,43 @@ class BaseTableView(QTableView):
                 QMessageBox.critical(self, title,
                      translate('DictEditor', "<b>Unable to save array</b>"
                                "<br><br>Error message:<br>%1").arg(str(error)))
+    def copy(self):
+        """Copy text to clipboard"""
+        clipboard = QApplication.clipboard()
+        clipl = []
+        for idx in self.selectedIndexes():
+            if not idx.isValid:
+                continue
+            clipl.append(unicode(self.delegate.get_value(idx)))
+        clipboard.setText(u'\n'.join(clipl))
+    
+    def import_from_string(self, text, title=None):
+        """Import data from string"""
+        data = self.model.get_data()
+        editor = ImportWizard(self, text, title=title,
+                              contents_title=translate("DictEditor",
+                                                       "Clipboard contents"),
+                              varname=fix_reference_name("data",
+                                                         blacklist=data.keys()))
+        if editor.exec_():
+            var_name, clip_data = editor.get_data()
+            self.new_value(var_name, clip_data)
+    
+    def paste(self):
+        """Import text/data/code from clipboard"""
+        clipboard = QApplication.clipboard()
+        cliptext = u""
+        if clipboard.mimeData().hasText():
+            cliptext = unicode(clipboard.text())
+        if cliptext.strip():
+            self.import_from_string(cliptext,
+                                    title=translate("DictEditor",
+                                                    "Import from clipboard"))
+        else:
+            QMessageBox.warning(self,
+                                translate("DictEditor", "Empty clipboard"),
+                                translate("DictEditor", "Nothing to be imported"
+                                          " from clipboard."))
         
 
 class DictEditorTableView(BaseTableView):
@@ -826,19 +923,6 @@ class DictEditorTableView(BaseTableView):
 
         self.setup_table()
         self.menu = self.setup_menu(truncate, minmax, inplace, collvalue)
-        self.copy_action = create_action(self,
-                                      translate("DictEditor", "Copy"),
-                                      icon=get_icon('editcopy.png'),
-                                      triggered=self.copy)                                      
-        self.paste_action = create_action(self,
-                                      translate("DictEditor", "Paste"),
-                                      icon=get_icon('editpaste.png'),
-                                      triggered=self.paste)
-        self.menu.insertAction(self.remove_action, self.copy_action)
-        self.menu.insertAction(self.remove_action, self.paste_action)
-        
-        self.empty_ws_menu = QMenu(self)
-        self.empty_ws_menu.addAction(self.paste_action)
     
     def remove_values(self, keys):
         """
@@ -883,61 +967,6 @@ class DictEditorTableView(BaseTableView):
         """Set table dict filter"""
         self.dictfilter = dictfilter
 
-    def copy(self):
-        """Copy text to clipboard"""
-        clipboard = QApplication.clipboard()
-        data = self.model.get_data()
-        clipl = []
-        for idx in self.selectedIndexes():
-            if not idx.isValid:
-                continue
-            _txt = u''
-            if isinstance(data,dict):
-                _txt = unicode(data.get(self.model.keys[idx.row()]))
-            else:
-                _txt = unicode(data[idx.row()])
-            clipl.append(_txt)
-        clipboard.setText(u'\n'.join(clipl))
-    
-    def import_from_string(self, text, title=None):
-        """Import data from string"""
-#        if isinstance(text, basestring):
-#            text = QString(text)
-        data = self.model.get_data()
-        varname_base = translate("DictEditor", "new")
-        try:
-            varname_base = str(varname_base)
-        except UnicodeEncodeError:
-            varname_base = unicode(varname_base)
-        get_varname = lambda index: varname_base + ("%03d" % index)
-        index = 0
-        while get_varname(index) in data:
-            index += 1
-        editor = ImportWizard(self, text, title=title,
-                              contents_title=translate("DictEditor",
-                                                       "Clipboard contents"),
-                              varname=get_varname(index))
-        if editor.exec_():
-            var_name, clip_data = editor.get_data()
-            data[var_name] = clip_data
-            self.set_data(data)
-    
-    def paste(self):
-        """Import text/data/code from clipboard"""
-        clipboard = QApplication.clipboard()
-        cliptext = u""
-        if clipboard.mimeData().hasText():
-            cliptext = unicode(clipboard.text())
-        if cliptext.strip():
-            self.import_from_string(cliptext,
-                                    title=translate("DictEditor",
-                                                    "Import from clipboard"))
-        else:
-            QMessageBox.warning(self,
-                                translate("DictEditor", "Empty clipboard"),
-                                translate("DictEditor", "Nothing to be imported"
-                                          " from clipboard."))
-
 
 class DictEditorWidget(QWidget):
     """Dictionary Editor Dialog"""
@@ -965,8 +994,12 @@ class DictEditor(QDialog):
     def __init__(self, data, title="", width=500,
                  readonly=False, icon='dictedit.png', remote=False):
         QDialog.__init__(self)
-        import copy
-        self.data_copy = copy.deepcopy(data)
+        if isinstance(data, dict):
+            # dictionnary
+            self.data_copy = data.copy()
+        else:
+            # list, tuple
+            self.data_copy = data[:]
         self.widget = DictEditorWidget(self, self.data_copy, title=title,
                                        readonly=readonly, remote=remote)
         
@@ -1012,8 +1045,7 @@ def dedit(seq):
     (instantiate a new QApplication if necessary,
     so it can be called directly from the interpreter)
     """
-    if QApplication.startingUp():
-        QApplication([])
+    _app = qapplication()
     dialog = DictEditor(seq)
     if dialog.exec_():
         return dialog.get_copy()
@@ -1100,33 +1132,42 @@ def globalsfilter(input_dict, itermax=-1, filters=None,
     return output_dict
 
 
-if __name__ == "__main__":
-    import numpy as np
+def get_test_data():
+    """Create test data"""
+    import numpy as np, PIL.Image
+    image = PIL.Image.fromarray(np.random.rand(100, 100))
     testdict = {'d': 1, 'a': np.random.rand(10, 10), 'b': [1, 2]}
     testdate = datetime.date(1945, 5, 8)
-    example = {'str': 'kjkj kj k j j kj k jkj',
-               'unicode': u'éù',
-               'list': [1, 3, [4, 5, 6], 'kjkj', None],
-               'tuple': ([1, testdate, testdict], 'kjkj', None),
-               'dict': testdict,
-               'float': 1.2233,
-               'array': np.random.rand(10, 10),
-               'empty_array': np.array([]),
-               'date': testdate,
-               'datetime': datetime.datetime(1945, 5, 8),
-               }
-    
-#    # Remote dict test:
-#    from spyderlib.widgets.monitor import make_remote_view
-#    remote = make_remote_view(example)
-#    from pprint import pprint
-#    pprint(remote)
-#    if QApplication.startingUp():
-#        QApplication([])
-#    dialog = DictEditor(remote, remote=True)
-#    if dialog.exec_():
-#        print dialog.get_copy()
-    
-    out = dedit(example)
+    return {'str': 'kjkj kj k j j kj k jkj',
+            'unicode': u'éù',
+            'list': [1, 3, [4, 5, 6], 'kjkj', None],
+            'tuple': ([1, testdate, testdict], 'kjkj', None),
+            'dict': testdict,
+            'float': 1.2233,
+            'array': np.random.rand(10, 10),
+            '1D-array': np.linspace(-10, 10),
+            'empty_array': np.array([]),
+            'image': image,
+            'date': testdate,
+            'datetime': datetime.datetime(1945, 5, 8),
+            }
+
+def test():
+    """Dictionary editor test"""
+    out = dedit( get_test_data() )
     print "out:", out
-    
\ No newline at end of file
+    
+def remote_editor_test():
+    """Remote dictionary editor test"""
+    from spyderlib.widgets.externalshell.globalsexplorer import get_settings
+    from spyderlib.widgets.externalshell.monitor import make_remote_view
+    remote = make_remote_view(get_test_data(), get_settings())
+    from pprint import pprint
+    pprint(remote)
+    _app = qapplication()
+    dialog = DictEditor(remote, remote=True)
+    if dialog.exec_():
+        print dialog.get_copy()
+
+if __name__ == "__main__":
+    test()
