@@ -25,7 +25,7 @@ except ImportError:
 from PyQt4.QtGui import (QDialog, QVBoxLayout, QLabel, QHBoxLayout, QDirModel,
                          QMessageBox, QInputDialog, QLineEdit, QMenu, QWidget,
                          QToolButton, QFileDialog, QToolBar, QTreeView, QDrag)
-from PyQt4.QtCore import Qt, SIGNAL, QMimeData, QSize, QDir, QStringList
+from PyQt4.QtCore import Qt, SIGNAL, QMimeData, QSize, QDir, QStringList, QUrl
 
 import os, sys, re
 import os.path as osp
@@ -37,7 +37,7 @@ STDOUT = sys.stdout
 from spyderlib.widgets.formlayout import fedit
 from spyderlib.utils.qthelpers import (get_std_icon, create_action, add_actions,
                                        translate)
-from spyderlib.utils import encoding
+from spyderlib.utils import encoding, rename_file, remove_file
 from spyderlib.config import get_icon
 
 
@@ -243,18 +243,29 @@ class ExplorerTreeWidget(DirView):
                 actions.append(browse_action if is_dir else edit_action)
             else:
                 actions.append(open_action)
-            actions += [delete_action, rename_action]
+            actions += [delete_action, rename_action, None]
             if is_dir and os.name == 'nt':
                 # Actions specific to Windows directories
                 actions.append( create_action(self,
                            translate('Explorer', "Open in Windows Explorer"),
                            icon="magnifier.png",
                            triggered=self.startfile) )
-        if os.name == 'nt':
-            actions.append( create_action(self,
-                       translate('Explorer', "Open command prompt here"),
-                       icon="cmdprompt.png",
-                       triggered=lambda cmd='cmd.exe': os.startfile(cmd)) )
+            if is_dir:
+                if os.name == 'nt':
+                    _title = translate('Explorer', "Open command prompt here")
+                else:
+                    _title = translate('Explorer', "Open terminal here")
+                action = create_action(self, _title, icon="cmdprompt.png",
+                                       triggered=lambda _fn=fname:
+                                       self.parent_widget.emit(
+                                       SIGNAL("open_terminal(QString)"), _fn))
+                actions.append(action)
+                _title = translate('Explorer', "Open Python interpreter here")
+                action = create_action(self, _title, icon="python.png",
+                                   triggered=lambda _fn=fname:
+                                   self.parent_widget.emit(
+                                   SIGNAL("open_interpreter(QString)"), _fn))
+                actions.append(action)
         if actions:
             actions.append(None)
         actions += self.common_actions
@@ -310,10 +321,10 @@ class ExplorerTreeWidget(DirView):
             
     def startDrag(self, dropActions):
         """Reimplement Qt Method - handle drag event"""
-        mimeData = QMimeData()
-        mimeData.setText(self.get_filename())
+        data = QMimeData()
+        data.setUrls([QUrl(self.get_filename())])
         drag = QDrag(self)
-        drag.setMimeData(mimeData)
+        drag.setMimeData(data)
         drag.exec_()
             
             
@@ -385,7 +396,7 @@ class ExplorerTreeWidget(DirView):
                 return
             try:
                 if osp.isfile(fname):
-                    os.remove(fname)
+                    remove_file(fname)
                 else:
                     os.rmdir(fname)
                 self.parent_widget.emit(SIGNAL("removed(QString)"), fname)
@@ -412,7 +423,7 @@ class ExplorerTreeWidget(DirView):
                 if path == fname:
                     return
                 try:
-                    os.rename(fname, path)
+                    rename_file(fname, path)
                     self.parent_widget.emit(SIGNAL("renamed(QString,QString)"),
                                              fname, path)
                 except EnvironmentError, error:
@@ -608,9 +619,15 @@ class Test(QDialog):
         self.connect(self.explorer, SIGNAL("open_parent_dir()"),
                      lambda: self.explorer.listwidget.refresh('..'))
 
-if __name__ == "__main__":
-    from PyQt4.QtGui import QApplication
-    app = QApplication([])
+
+def test():
+    """Run file/directory explorer test"""
+    from spyderlib.utils.qthelpers import qapplication
+    app = qapplication()
     test = Test()
     test.show()
     sys.exit(app.exec_())
+    
+if __name__ == "__main__":
+    test()
+    
\ No newline at end of file
