@@ -1,6 +1,15 @@
 # -*- coding: utf-8 -*-
 # Spyder's ExternalPythonShell sitecustomize
 
+try:
+    import locale, win32console, pywintypes
+    _t, _cp = locale.getdefaultlocale('LANG')
+    _cp = int(_cp[2:])
+    win32console.SetConsoleCP(_cp)
+    win32console.SetConsoleOutputCP(_cp)
+except (ImportError, ValueError, TypeError, pywintypes.error):
+    # Pywin32 is not installed or Code page number in locale is not valid
+    pass
 
 # Set standard outputs encoding:
 # (otherwise, for example, print u"é" will fail)
@@ -45,13 +54,14 @@ def displayhook(obj):
 
 sys.displayhook = displayhook
 
-from PyQt4.QtCore import pyqtRemoveInputHook
-pyqtRemoveInputHook()
+if os.name == 'nt':
+    from PyQt4.QtCore import pyqtRemoveInputHook
+    pyqtRemoveInputHook()
 
-# Restoring original PYTHONPATH
-try:
-    os.environ['PYTHONPATH'] = os.environ['OLD_PYTHONPATH']
-    del os.environ['OLD_PYTHONPATH']
-except KeyError:
-    if os.environ.get('PYTHONPATH') is not None:
-        del os.environ['PYTHONPATH']
+## Restoring original PYTHONPATH
+#try:
+#    os.environ['PYTHONPATH'] = os.environ['OLD_PYTHONPATH']
+#    del os.environ['OLD_PYTHONPATH']
+#except KeyError:
+#    if os.environ.get('PYTHONPATH') is not None:
+#        del os.environ['PYTHONPATH']
