@@ -46,6 +46,13 @@ def abspardir(path):
     """Return absolute parent dir"""
     return osp.abspath(osp.join(path, os.pardir))
 
+def get_common_path(pathlist):
+    common = osp.commonprefix(pathlist)
+    if len(common) > 1:
+        if not osp.isdir(common):
+            common = osp.dirname(common)
+        return osp.abspath(common)
+
 def is_hg_installed():
     return is_program_installed('hg.exe' if os.name == 'nt' else 'hg')
 
@@ -126,10 +133,20 @@ class SearchThread(QThread):
     def __init__(self, parent):
         QThread.__init__(self, parent)
         self.mutex = QMutex()
-        self.stopped = False
+        self.stopped = None
         self.results = None
+        self.pathlist = None
         self.nb = None
         self.error_flag = None
+        self.rootpath = None
+        self.python_path = None
+        self.hg_manifest = None
+        self.include = None
+        self.exclude = None
+        self.texts = None
+        self.text_re = None
+        self.completed = None
+        self.get_pythonpath_callback = None
         
     def initialize(self, path, python_path, hg_manifest,
                    include, exclude, texts, text_re):
@@ -144,12 +161,13 @@ class SearchThread(QThread):
         self.completed = False
         
     def run(self):
+        self.filenames = []
         if self.hg_manifest:
             ok = self.find_files_in_hg_manifest()
         elif self.python_path:
             ok = self.find_files_in_python_path()
         else:
-            ok = self.find_files_in_path()
+            ok = self.find_files_in_path(self.rootpath)
         if ok:
             self.find_string_in_files()
         self.stop()
@@ -160,60 +178,76 @@ class SearchThread(QThread):
             self.stopped = True
 
     def find_files_in_python_path(self):
-        self.filenames = []
-        directories = [path for path in sys.path \
-                       if not path.startswith(sys.prefix)]
-        for path in directories:
-            if not path:
-                path = os.getcwdu()
-            with QMutexLocker(self.mutex):
-                if self.stopped:
-                    return False
-            dirname = osp.dirname(path)
-            if re.search(self.exclude, dirname+os.sep):
-                continue
-            filename = osp.dirname(path)
-            if re.search(self.exclude, filename):
-                continue
-            if re.search(self.include, filename):
-                self.filenames.append(path)
-        return True
+        pathlist = os.environ['PYTHONPATH'].split(os.pathsep)
+        if self.get_pythonpath_callback is not None:
+            pathlist += self.get_pythonpath_callback()
+        if os.name == "nt":
+            # The following avoid doublons on Windows platforms:
+            # (e.g. "d:\Python" in PYTHONPATH environment variable,
+            #  and  "D:\Python" in Spyder's python path would lead 
+            #  to two different search folders)
+            winpathlist = []
+            lcpathlist = []
+            for path in pathlist:
+                lcpath = osp.normcase(path)
+                if lcpath not in lcpathlist:
+                    lcpathlist.append(lcpath)
+                    winpathlist.append(path)
+            pathlist = winpathlist
+        for path in set(pathlist):
+            if osp.isdir(path):
+                ok = self.find_files_in_path(path)
+                if not ok:
+                    break
+        return ok
 
     def find_files_in_hg_manifest(self):
         p = Popen(['hg', 'manifest'], stdout=PIPE,
                   cwd=self.rootpath, shell=True)
-        self.filenames = []
         hgroot = get_hg_root(self.rootpath)
+        self.pathlist = [hgroot]
         for path in p.stdout.read().splitlines():
             with QMutexLocker(self.mutex):
                 if self.stopped:
                     return False
             dirname = osp.dirname(path)
-            if re.search(self.exclude, dirname+os.sep):
-                continue
-            filename = osp.basename(path)
-            if re.search(self.exclude, filename):
-                continue
-            if re.search(self.include, filename):
-                self.filenames.append(osp.join(hgroot, path))
-        return True
-    
-    def find_files_in_path(self):
-        self.filenames = []
-        for path, dirs, files in os.walk(self.rootpath):
-            with QMutexLocker(self.mutex):
-                if self.stopped:
-                    return False
-            for d in dirs[:]:
-                dirname = os.path.join(path, d)
+            try:
                 if re.search(self.exclude, dirname+os.sep):
-                    dirs.remove(d)
-            for f in files:
-                filename = os.path.join(path, f)
+                    continue
+                filename = osp.basename(path)
                 if re.search(self.exclude, filename):
                     continue
                 if re.search(self.include, filename):
-                    self.filenames.append(filename)
+                    self.filenames.append(osp.join(hgroot, path))
+            except re.error:
+                self.error_flag = translate("FindInFiles",
+                                            "invalid regular expression")
+                return False
+        return True
+    
+    def find_files_in_path(self, path):
+        if self.pathlist is None:
+            self.pathlist = []
+        self.pathlist.append(path)
+        for path, dirs, files in os.walk(path):
+            with QMutexLocker(self.mutex):
+                if self.stopped:
+                    return False
+            try:
+                for d in dirs[:]:
+                    dirname = os.path.join(path, d)
+                    if re.search(self.exclude, dirname+os.sep):
+                        dirs.remove(d)
+                for f in files:
+                    filename = os.path.join(path, f)
+                    if re.search(self.exclude, filename):
+                        continue
+                    if re.search(self.include, filename):
+                        self.filenames.append(filename)
+            except re.error:
+                self.error_flag = translate("FindInFiles",
+                                            "invalid regular expression")
+                return False
         return True
         
     def find_string_in_files(self):
@@ -254,10 +288,13 @@ class SearchThread(QThread):
             except IOError, (_errno, _strerror):
                 self.error_flag = translate("FindInFiles",
                                   "permission denied errors were encountered")
+            except re.error:
+                self.error_flag = translate("FindInFiles",
+                                            "invalid regular expression")
         self.completed = True
     
     def get_results(self):
-        return self.results, self.nb, self.error_flag
+        return self.results, self.pathlist, self.nb, self.error_flag
 
 
 class FindOptions(QWidget):
@@ -265,8 +302,9 @@ class FindOptions(QWidget):
     Find widget with options
     """
     def __init__(self, parent, search_text, search_text_regexp, search_path,
-                 include, include_regexp, exclude, exclude_regexp,
-                 supported_encodings):
+                 include, include_idx, include_regexp,
+                 exclude, exclude_idx, exclude_regexp,
+                 supported_encodings, in_python_path, more_options):
         QWidget.__init__(self, parent)
         
         if search_path is None:
@@ -287,12 +325,16 @@ class FindOptions(QWidget):
         hlayout1 = QHBoxLayout()
         self.search_text = PatternComboBox(self, search_text,
                                     translate('FindInFiles', "Search pattern"))
-        search_label = QLabel(translate('FindInFiles', "Search text:"))
-        search_label.setBuddy(self.search_text)
         self.edit_regexp = create_toolbutton(self, get_icon("advanced.png"),
                          tip=translate('FindInFiles', "Regular expression"))
         self.edit_regexp.setCheckable(True)
         self.edit_regexp.setChecked(search_text_regexp)
+        self.more_widgets = ()
+        self.more_options = create_toolbutton(self,
+                                              toggled=self.toggle_more_options)
+        self.more_options.setCheckable(True)
+        self.more_options.setChecked(more_options)
+        
         self.ok_button = create_toolbutton(self,
                                 text=translate('FindInFiles', "Search"),
                                 triggered=lambda: self.emit(SIGNAL('find()')),
@@ -305,14 +347,17 @@ class FindOptions(QWidget):
                                 icon=get_icon("terminate.png"),
                                 tip=translate('FindInFiles', "Stop search"))
         self.stop_button.setEnabled(False)
-        for widget in [search_label, self.search_text, self.edit_regexp,
-                       self.ok_button, self.stop_button]:
+        for widget in [self.search_text, self.edit_regexp,
+                       self.ok_button, self.stop_button, self.more_options]:
             hlayout1.addWidget(widget)
 
         # Layout 2
         hlayout2 = QHBoxLayout()
         self.include_pattern = PatternComboBox(self, include,
                         translate('FindInFiles', "Included filenames pattern"))
+        if include_idx is not None and include_idx >= 0 \
+           and include_idx < self.include_pattern.count():
+            self.include_pattern.setCurrentIndex(include_idx)
         self.include_regexp = create_toolbutton(self, get_icon("advanced.png"),
                                             tip=translate('FindInFiles',
                                                           "Regular expression"))
@@ -322,6 +367,9 @@ class FindOptions(QWidget):
         include_label.setBuddy(self.include_pattern)
         self.exclude_pattern = PatternComboBox(self, exclude,
                         translate('FindInFiles', "Excluded filenames pattern"))
+        if exclude_idx is not None and exclude_idx >= 0 \
+           and exclude_idx < self.exclude_pattern.count():
+            self.exclude_pattern.setCurrentIndex(exclude_idx)
         self.exclude_regexp = create_toolbutton(self, get_icon("advanced.png"),
                                             tip=translate('FindInFiles',
                                                           "Regular expression"))
@@ -337,9 +385,9 @@ class FindOptions(QWidget):
 
         # Layout 3
         hlayout3 = QHBoxLayout()
-        searchin_label = QLabel(translate('FindInFiles', "Search in:"))
         self.python_path = QRadioButton(translate('FindInFiles',
                                         "PYTHONPATH"), self)
+        self.python_path.setChecked(in_python_path)
         self.python_path.setToolTip(translate('FindInFiles',
                           "Search in all directories listed in sys.path which"
                           " are outside the Python installation directory"))        
@@ -348,10 +396,8 @@ class FindOptions(QWidget):
         self.detect_hg_repository()
         self.hg_manifest.setToolTip(translate('FindInFiles',
                               "Search in current directory hg repository"))
-        searchin_label.setBuddy(self.hg_manifest)
-        self.custom_dir = QRadioButton(translate('FindInFiles',
-                                                 "Directory:"), self)
-        self.custom_dir.setChecked(True)
+        self.custom_dir = QRadioButton(translate('FindInFiles', "Here:"), self)
+        self.custom_dir.setChecked(not in_python_path)
         self.dir_combo = PathComboBox(self)
         self.dir_combo.addItems(search_path)
         self.dir_combo.setToolTip(translate('FindInFiles',
@@ -366,18 +412,42 @@ class FindOptions(QWidget):
                                    tip=translate('FindInFiles',
                                                  'Browse a search directory'),
                                    triggered=self.select_directory)
-        for widget in [searchin_label, self.python_path, self.hg_manifest,
-                       self.custom_dir, self.dir_combo, browse]:
+        for widget in [self.python_path, self.hg_manifest, self.custom_dir,
+                       self.dir_combo, browse]:
             hlayout3.addWidget(widget)
             
+        self.connect(self.search_text, SIGNAL("valid(bool)"),
+                     lambda valid: self.emit(SIGNAL('find()')))
+        self.connect(self.include_pattern, SIGNAL("valid(bool)"),
+                     lambda valid: self.emit(SIGNAL('find()')))
+        self.connect(self.exclude_pattern, SIGNAL("valid(bool)"),
+                     lambda valid: self.emit(SIGNAL('find()')))
+        self.connect(self.dir_combo, SIGNAL("valid(bool)"),
+                     lambda valid: self.emit(SIGNAL('find()')))
+            
         vlayout = QVBoxLayout()
         vlayout.addLayout(hlayout1)
         vlayout.addLayout(hlayout2)
         vlayout.addLayout(hlayout3)
+        self.more_widgets = (hlayout2, hlayout3)
+        self.toggle_more_options(more_options)
         self.setLayout(vlayout)
                 
         self.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Minimum)
         
+    def toggle_more_options(self, state):
+        for layout in self.more_widgets:
+            for index in range(layout.count()):
+                layout.itemAt(index).widget().setVisible(state)
+        if state:
+            icon_name = 'options_less.png'
+            tip = translate('FindInFiles', 'Hide advanced options')
+        else:
+            icon_name = 'options_more.png'
+            tip = translate('FindInFiles', 'Show advanced options')
+        self.more_options.setIcon(get_icon(icon_name))
+        self.more_options.setToolTip(tip)
+        
     def update_combos(self):
         self.search_text.lineEdit().emit(SIGNAL('returnPressed()'))
         self.include_pattern.lineEdit().emit(SIGNAL('returnPressed()'))
@@ -403,7 +473,7 @@ class FindOptions(QWidget):
             return
         try:
             texts = [str(utext)]
-        except UnicodeDecodeError:
+        except UnicodeEncodeError:
             texts = []
             for encoding in self.supported_encodings:
                 try:
@@ -432,11 +502,15 @@ class FindOptions(QWidget):
                            for index in range(self.dir_combo.count())]
             include = [unicode(self.include_pattern.itemText(index)) \
                        for index in range(self.include_pattern.count())]
+            include_idx = self.include_pattern.currentIndex()
             exclude = [unicode(self.exclude_pattern.itemText(index)) \
                        for index in range(self.exclude_pattern.count())]
+            exclude_idx = self.exclude_pattern.currentIndex()
+            more_options = self.more_options.isChecked()
             return (search_text, text_re, search_path,
-                    include, include_re,
-                    exclude, exclude_re)
+                    include, include_idx, include_re,
+                    exclude, exclude_idx, exclude_re,
+                    python_path, more_options)
         else:
             return (path, python_path, hg_manifest,
                     include, exclude, texts, text_re)
@@ -474,34 +548,34 @@ class FindOptions(QWidget):
 class ResultsBrowser(OneColumnTree):
     def __init__(self, parent):
         OneColumnTree.__init__(self, parent)
+        self.search_text = None
         self.results = None
         self.nb = None
         self.error_flag = None
+        self.completed = None
         self.data = None
         self.set_title('')
-        self.root_item = None
+        self.root_items = None
         
     def activated(self):
         itemdata = self.data.get(self.currentItem())
         if itemdata is not None:
             filename, lineno = itemdata
-            self.parent().emit(SIGNAL("edit_goto(QString,int)"),
-                               filename, lineno)
+            self.parent().emit(SIGNAL("edit_goto(QString,int,QString)"),
+                               filename, lineno, self.search_text)
         
-    def set_results(self, search_text, results, nb, error_flag):
+    def set_results(self, search_text, results, pathlist, nb,
+                    error_flag, completed):
         self.search_text = search_text
         self.results = results
+        self.pathlist = pathlist
         self.nb = nb
         self.error_flag = error_flag
+        self.completed = completed
         self.refresh()
         if not self.error_flag and self.nb:
             self.restore()
         
-    def restore(self):
-        self.collapseAll()
-        if self.root_item is not None:
-            self.root_item.setExpanded(True)
-        
     def refresh(self):
         """
         Refreshing search results panel
@@ -520,32 +594,41 @@ class ResultsBrowser(OneColumnTree):
                     text_files += 's'
                 text = "%d %s %d %s" % (self.nb, text_matches,
                                         nb_files, text_files)
-            if self.error_flag:
-                text += ' (' + self.error_flag + ')'
+        if self.error_flag:
+            text += ' (' + self.error_flag + ')'
+        elif self.results is not None and not self.completed:
+            text += ' (' + translate('FindInFiles', 'interrupted') + ')'
         self.set_title(title+text)
         self.clear()
         self.data = {}
         
-        if self.results is None: # First search interrupted
+        if not self.results: # First search interrupted *or* No result
             return
-        
-        # Root path
-        root_path = None
-        dir_set = set()        
-        for filename in self.results:
-            dirname = osp.dirname(filename)
+
+        # Directory set
+        dir_set = set()
+        for filename in sorted(self.results.keys()):
+            dirname = osp.abspath(osp.dirname(filename))
             dir_set.add(dirname)
-            if root_path is None:
-                root_path = dirname
+                
+        # Root path
+        root_path_list = None
+        _common = get_common_path(list(dir_set))
+        if _common is not None:
+            root_path_list = [_common]
+        else:
+            _common = get_common_path(self.pathlist)
+            if _common is not None:
+                root_path_list = [_common]
             else:
-                while root_path not in dirname:
-                    root_path = abspardir(root_path)
-        if root_path is None:
+                root_path_list = self.pathlist
+        if not root_path_list:
             return
-        dir_set.add(root_path)
+        for _root_path in root_path_list:
+            dir_set.add(_root_path)
         # Populating tree: directories
         def create_dir_item(dirname, parent):
-            if dirname != root_path:
+            if dirname not in root_path_list:
                 displayed_name = osp.basename(dirname)
             else:
                 displayed_name = dirname
@@ -554,7 +637,7 @@ class ResultsBrowser(OneColumnTree):
             return item
         dirs = {}
         for dirname in sorted(list(dir_set)):
-            if dirname == root_path:
+            if dirname in root_path_list:
                 parent = self
             else:
                 parent_dirname = abspardir(dirname)
@@ -562,6 +645,9 @@ class ResultsBrowser(OneColumnTree):
                 if parent is None:
                     # This is related to directories which contain found
                     # results only in some of their children directories
+                    if osp.commonprefix([dirname]+root_path_list):
+                        # create new root path
+                        pass
                     items_to_create = []
                     while dirs.get(parent_dirname) is None:
                         items_to_create.append(parent_dirname)
@@ -573,7 +659,7 @@ class ResultsBrowser(OneColumnTree):
                     parent_dirname = abspardir(dirname)
                     parent = dirs[parent_dirname]
             dirs[dirname] = create_dir_item(dirname, parent)
-        self.root_item = dirs[root_path]
+        self.root_items = [dirs[_root_path] for _root_path in root_path_list]
         # Populating tree: files
         for filename in sorted(self.results.keys()):
             parent_item = dirs[osp.dirname(filename)]
@@ -584,6 +670,12 @@ class ResultsBrowser(OneColumnTree):
                            ["%d (%d): %s" % (lineno, colno, line.rstrip())])
                 item.setIcon(0, get_icon('arrow.png'))
                 self.data[item] = (filename, lineno)
+        # Removing empty directories
+        top_level_items = [self.topLevelItem(index)
+                           for index in range(self.topLevelItemCount())]
+        for item in top_level_items:
+            if not item.childCount():
+                self.takeTopLevelItem(self.indexOfTopLevelItem(item))
 
 
 class FindInFilesWidget(QWidget):
@@ -593,37 +685,47 @@ class FindInFilesWidget(QWidget):
     def __init__(self, parent,
                  search_text = r"# ?TODO|# ?FIXME|# ?XXX",
                  search_text_regexp=True, search_path=None,
-                 include=".", include_regexp=True,
-                 exclude=r"\.pyc$|\.orig$|\.hg|\.svn", exclude_regexp=True,
-                 supported_encodings=("utf-8", "iso-8859-1", "cp1252")):
+                 include=[".", ".py"], include_idx=None, include_regexp=True,
+                 exclude=r"\.pyc$|\.orig$|\.hg|\.svn", exclude_idx=None,
+                 exclude_regexp=True,
+                 supported_encodings=("utf-8", "iso-8859-1", "cp1252"),
+                 in_python_path=False, more_options=False):
         QWidget.__init__(self, parent)
+        
+        self.setWindowTitle(translate('FindInFiles', 'Find in files'))
 
         self.search_thread = SearchThread(self)
-        self.connect(self.search_thread, SIGNAL("finished()"),
+        self.connect(self.search_thread, SIGNAL("finished(bool)"),
                      self.search_complete)
         
         self.find_options = FindOptions(self, search_text, search_text_regexp,
-                                        search_path, include, include_regexp,
-                                        exclude, exclude_regexp,
-                                        supported_encodings)
+                                        search_path,
+                                        include, include_idx, include_regexp,
+                                        exclude, exclude_idx, exclude_regexp,
+                                        supported_encodings, in_python_path,
+                                        more_options)
         self.connect(self.find_options, SIGNAL('find()'), self.find)
         self.connect(self.find_options, SIGNAL('stop()'), self.stop)
         
         self.result_browser = ResultsBrowser(self)
         
-        collapse_btn = create_toolbutton(self, get_icon("collapse.png"),
-                                 tip=translate('FindInFiles', "Collapse all"),
-                                 triggered=self.result_browser.collapseAll)
-        expand_btn = create_toolbutton(self, get_icon("expand.png"),
-                                 tip=translate('FindInFiles', "Expand all"),
-                                 triggered=self.result_browser.expandAll)
-        restore_btn = create_toolbutton(self, get_icon("restore.png"),
-                                 tip=translate('FindInFiles',
-                                               "Restore original tree layout"),
-                                 triggered=self.result_browser.restore)
+        collapse_btn = create_toolbutton(self, text_beside_icon=False)
+        collapse_btn.setDefaultAction(self.result_browser.collapse_all_action)
+        expand_btn = create_toolbutton(self, text_beside_icon=False)
+        expand_btn.setDefaultAction(self.result_browser.expand_all_action)
+        restore_btn = create_toolbutton(self, text_beside_icon=False)
+        restore_btn.setDefaultAction(self.result_browser.restore_action)
+#        collapse_sel_btn = create_toolbutton(self, text_beside_icon=False)
+#        collapse_sel_btn.setDefaultAction(
+#                                self.result_browser.collapse_selection_action)
+#        expand_sel_btn = create_toolbutton(self, text_beside_icon=False)
+#        expand_sel_btn.setDefaultAction(
+#                                self.result_browser.expand_selection_action)
+        
         btn_layout = QVBoxLayout()
         btn_layout.setAlignment(Qt.AlignTop)
         for widget in [collapse_btn, expand_btn, restore_btn]:
+#                       collapse_sel_btn, expand_sel_btn]:
             btn_layout.addWidget(widget)
         
         hlayout = QHBoxLayout()
@@ -639,6 +741,9 @@ class FindInFilesWidget(QWidget):
             
     def set_search_text(self, text):
         self.find_options.set_search_text(text)
+        
+    def set_pythonpath_callback(self, callback):
+        self.search_thread.get_pythonpath_callback = callback
 
     def find(self):
         """Call the find function"""
@@ -654,23 +759,26 @@ class FindInFilesWidget(QWidget):
         if self.search_thread.isRunning():
             self.search_thread.stop()
             
-    def search_complete(self):
+    def search_complete(self, completed):
         self.find_options.ok_button.setEnabled(True)
         self.find_options.stop_button.setEnabled(False)
         found = self.search_thread.get_results()
         if found is not None:
-            results, nb, error_flag = found
+            results, pathlist, nb, error_flag = found
             search_text = unicode( self.find_options.search_text.currentText() )
-            self.result_browser.set_results(search_text, results,
-                                            nb, error_flag)
+            self.result_browser.set_results(search_text, results, pathlist,
+                                            nb, error_flag, completed)
             self.result_browser.show()
             
             
-if __name__ == '__main__':
-    from PyQt4.QtGui import QApplication
-    app = QApplication([])
-    
+def test():
+    """Run Find in Files widget test"""
+    from spyderlib.utils.qthelpers import qapplication
+    app = qapplication()
     widget = FindInFilesWidget(None)
     widget.show()
-    
     sys.exit(app.exec_())
+    
+if __name__ == '__main__':
+    test()
+    
\ No newline at end of file
