@@ -106,11 +106,14 @@ class InteractiveShell(PythonShellWidget):
     p1 = ">>> "
     p2 = "... "
     def __init__(self, parent=None, namespace=None, commands=None, message="",
-                 font=None, debug=False, exitfunc=None, profile=False):
+                 max_line_count=300, font=None, debug=False, exitfunc=None,
+                 profile=False):
         PythonShellWidget.__init__(self, parent,
                                    get_conf_path('.history_ic.py'),
                                    debug, profile)
         
+        self.setMaximumBlockCount(max_line_count)
+        
         if font is not None:
             self.set_font(font)
         
@@ -135,9 +138,9 @@ class InteractiveShell(PythonShellWidget):
         
         # Code completion / calltips
         getcfg = lambda option: CONF.get('shell', option)
-        case_sensitive = getcfg('autocompletion/case-sensitivity')
-        show_single = getcfg('autocompletion/select-single')
-        from_document = getcfg('autocompletion/from-document')
+        case_sensitive = getcfg('codecompletion/case-sensitivity')
+        show_single = getcfg('codecompletion/select-single')
+        from_document = getcfg('codecompletion/from-document')
         self.setup_code_completion(case_sensitive, show_single, from_document)
         
         # keyboard events management
@@ -197,6 +200,9 @@ class InteractiveShell(PythonShellWidget):
         # First prompt
         self.new_prompt(self.p1)
         self.emit(SIGNAL("refresh()"))
+        
+        # Installing (if enabled) Rollback Importer
+        self.interpreter.install_rollback_importer()
 
         return self.interpreter
 
@@ -385,7 +391,7 @@ has the same effect as typing a particular string at the help> prompt.
         """
         for line in lines.splitlines():
             stripped_line = line.strip()
-            if len(stripped_line) == 0 or stripped_line.startswith('#'):
+            if stripped_line.startswith('#'):
                 continue
             self.write(line+os.linesep, flush=True)
             self.execute_command(line+"\n")
@@ -429,12 +435,16 @@ has the same effect as typing a particular string at the help> prompt.
         special_pattern = r"^%s (?:r\')?(?:u\')?\"?\'?([a-zA-Z0-9_\.]+)"
         run_match = re.match(special_pattern % 'run', cmd)
         help_match = re.match(r'^([a-zA-Z0-9_\.]+)\?$', cmd)
+        cd_match = re.match(r"^\!cd \"?\'?([a-zA-Z0-9_ \.]+)", cmd)
         if help_match:
             cmd = 'help(%s)' % help_match.group(1)
         # run command
         elif run_match:
             filename = guess_filename(run_match.groups()[0])
-            cmd = 'execfile(r"%s")' % filename
+            cmd = 'runfile(r"%s", args=None)' % filename
+        # !cd system command
+        elif cd_match:
+            cmd = 'import os; os.chdir(r"%s")' % cd_match.groups()[0].strip()
         # -- End of Special commands type I
             
         # -- Special commands type II
@@ -506,6 +516,14 @@ has the same effect as typing a particular string at the help> prompt.
         obj, valid = self._eval(objtxt)
         if valid:
             return getobjdir(obj)
+        
+    def get_globals_keys(self):
+        """Return shell globals() keys"""
+        return self.interpreter.namespace.keys()
+        
+    def get_cdlistdir(self):
+        """Return shell current directory list dir"""
+        return os.listdir(os.getcwdu())
                 
     def iscallable(self, objtxt):
         """Is object callable?"""
