@@ -33,7 +33,7 @@ import os.path as osp
 STDOUT = sys.stdout
 
 # Local imports
-from spyderlib.utils.programs import is_program_installed
+from spyderlib.utils import programs
 from spyderlib.utils.qthelpers import create_toolbutton, translate
 from spyderlib.config import get_icon, get_conf_path
 from spyderlib.widgets import OneColumnTree
@@ -42,10 +42,10 @@ from spyderlib.widgets.comboboxes import (PythonModulesComboBox,
                                           is_module_or_package)
 
 
-PYLINT_PATH = 'pylint.bat' if os.name == 'nt' else 'pylint'
+PYLINT_PATH = programs.get_nt_program_name('pylint')
 
 def is_pylint_installed():
-    return is_program_installed(PYLINT_PATH)
+    return programs.is_program_installed(PYLINT_PATH)
 
 class ResultsTree(OneColumnTree):
     def __init__(self, parent):
@@ -59,7 +59,8 @@ class ResultsTree(OneColumnTree):
         data = self.data.get(self.currentItem())
         if data is not None:
             fname, lineno = data
-            self.parent().emit(SIGNAL("edit_goto(QString,int)"), fname, lineno)
+            self.parent().emit(SIGNAL("edit_goto(QString,int,QString)"),
+                               fname, lineno, '')
         
     def set_results(self, filename, results):
         self.filename = filename
@@ -130,6 +131,8 @@ class PylintWidget(QWidget):
     def __init__(self, parent, max_entries=100):
         QWidget.__init__(self, parent)
         
+        self.setWindowTitle("Pylint")
+        
         self.output = None
         self.error_output = None
         
@@ -199,9 +202,17 @@ class PylintWidget(QWidget):
             for widget in (self.treewidget, self.filecombo,
                            self.start_button, self.stop_button):
                 widget.setDisabled(True)
-            text = translate('Pylint', 'Please install <b>pylint</b>:')
-            url = 'http://www.logilab.fr'
-            text += ' <a href=%s>%s</a>' % (url, url)
+            if os.name == 'nt' \
+               and programs.is_module_installed("pylint"):
+                # Pylint is installed but pylint script is not in PATH
+                # (AFAIK, could happen only on Windows)
+                text = translate('Pylint',
+                     'Pylint script was not found. Please add "%s" to PATH.')
+                text = unicode(text) % osp.join(sys.prefix, "Scripts")
+            else:
+                text = translate('Pylint', 'Please install <b>pylint</b>:')
+                url = 'http://www.logilab.fr'
+                text += ' <a href=%s>%s</a>' % (url, url)
             self.ratelabel.setText(text)
         else:
             self.show_data()
@@ -251,7 +262,7 @@ class PylintWidget(QWidget):
         filename = osp.abspath(filename)
         index, _data = self.get_data(filename)
         if index is not None:
-            self.data.pop(index)
+            self.data.pop(index+1)
         self.data.append( (filename, data) )
         self.save()
         
@@ -426,12 +437,14 @@ class PylintWidget(QWidget):
         self.datelabel.setText(date_text)
 
 
-if __name__ == '__main__':
-    from PyQt4.QtGui import QApplication
-    app = QApplication([])
-    
+def test():
+    """Run pylint widget test"""
+    from spyderlib.utils.qthelpers import qapplication
+    app = qapplication()
     widget = PylintWidget(None)
     widget.show()
     widget.analyze(__file__)
-    
     sys.exit(app.exec_())
+    
+if __name__ == '__main__':
+    test()
