@@ -15,7 +15,7 @@ import sys, os, time
 import os.path as osp
 
 from PyQt4.QtGui import (QMenu, QApplication, QCursor, QToolTip, QKeySequence,
-                         QFileDialog, QMessageBox)
+                         QFileDialog, QMessageBox, QMouseEvent)
 from PyQt4.QtCore import Qt, QString, QCoreApplication, SIGNAL, pyqtProperty
 
 # For debugging purpose:
@@ -30,10 +30,10 @@ from spyderlib.utils.dochelpers import getobj
 from spyderlib.utils.qthelpers import (keybinding, create_action, add_actions,
                                        restore_keyevent, translate)
 from spyderlib.widgets.shellhelpers import get_error_match
-# Shell widgets are based on QTextEdit (defined in 'qtebase' module) instead of
-# QsciScintilla (defined in 'qscibase' module - used in the editor widgets)
-# because QTextEdit is much faster:
-from spyderlib.widgets.qtebase import ConsoleBaseWidget
+# Shell widgets are based on QPlainTextEdit (defined in 'qtebase' module) 
+# instead of QsciScintilla (defined in 'qscibase' module - used in the editor 
+# widgets) because QPlainTextEdit is much faster:
+from spyderlib.widgets.qteditor.qtebase import ConsoleBaseWidget
 
 
 HISTORY_FILENAMES = []
@@ -51,7 +51,7 @@ class ShellBaseWidget(ConsoleBaseWidget):
         parent : specifies the parent widget
         """
         ConsoleBaseWidget.__init__(self, parent)
-        
+                
         # Prompt position: tuple (line, index)
         self.current_prompt_pos = None
         self.new_input_line = True
@@ -126,14 +126,26 @@ class ShellBaseWidget(ConsoleBaseWidget):
                                           "Save current history log (i.e. all "
                                           "inputs and outputs) in a text file"),
                                     triggered=self.save_historylog)
+        self.delete_action = create_action(self,
+                                    translate("ShellBaseWidget", "Delete"),
+                                    shortcut=keybinding('Delete'),
+                                    icon=get_icon('editdelete.png'),
+                                    triggered=self.delete)
+        selectall_action = create_action(self,
+                                    translate("ShellBaseWidget", "Select all"),
+                                    shortcut=keybinding('SelectAll'),
+                                    icon=get_icon('selectall.png'),
+                                    triggered=self.selectAll)
         add_actions(self.menu, (self.cut_action, self.copy_action,
-                                paste_action, None, save_action) )
+                                paste_action, self.delete_action, None,
+                                selectall_action, None, save_action) )
           
     def contextMenuEvent(self, event):
         """Reimplement Qt method"""
         state = self.hasSelectedText()
         self.copy_action.setEnabled(state)
         self.cut_action.setEnabled(state)
+        self.delete_action.setEnabled(state)
         self.menu.popup(event.globalPos())
         event.accept()        
         
@@ -308,6 +320,7 @@ class ShellBaseWidget(ConsoleBaseWidget):
         
         # Is cursor on the last line? and after prompt?
         if len(text):
+            #XXX: Shouldn't it be: `if len(unicode(text).strip(os.linesep))` ?
             if self.hasSelectedText():
                 self.check_selection()
             self.restrict_cursor_position(self.current_prompt_pos, 'eof')
@@ -389,7 +402,7 @@ class ShellBaseWidget(ConsoleBaseWidget):
         elif key in (Qt.Key_PageUp, Qt.Key_PageDown):
             #XXX: Find a way to do this programmatically instead of calling
             # widget keyhandler (this won't work if the *event* is coming from
-            # the event queue - i.e. when the busy buffer will be implemented)
+            # the event queue - i.e. if the busy buffer is ever implemented)
             ConsoleBaseWidget.keyPressEvent(self, event)
 
         elif key == Qt.Key_Escape:
@@ -411,6 +424,10 @@ class ShellBaseWidget(ConsoleBaseWidget):
         elif key == Qt.Key_Y and ctrl:
             self.redo()
             event.accept()
+            
+        elif key == Qt.Key_A and ctrl:
+            self.selectAll()
+            event.accept()
                 
         elif key == Qt.Key_Question and not self.hasSelectedText():
             self._key_question(text)
@@ -433,7 +450,7 @@ class ShellBaseWidget(ConsoleBaseWidget):
             self.zoomOut()
             event.accept()
 
-        elif text.length():
+        elif text.length() and not self.isReadOnly():
             self.hist_wholeline = False
             self.insert_text(text)
             self._key_other(text)
@@ -446,8 +463,8 @@ class ShellBaseWidget(ConsoleBaseWidget):
                 
     #------ Key handlers
     def _key_enter(self):
-        self.insert_text('\n', at_end=True)
         command = self.input_buffer
+        self.insert_text('\n', at_end=True)
         self.on_enter(command)
         self.flush()
     def _key_other(self, text):
@@ -602,15 +619,9 @@ class ShellBaseWidget(ConsoleBaseWidget):
         Insert text at the current cursor position
         or at the end of the command line
         """
-        if self.isReadOnly():
-            return
-        if error and text.startswith('  File "<'):
-            # Avoid printing 'File <console> [...]' which is related to the
-            # code.InteractiveConsole Python interpreter emulation
-            return
         if at_end:
             # Insert text at the end of the command line
-            self.append_text_to_pythonshell(text, error, prompt)
+            self.append_text_to_shell(text, error, prompt)
         else:
             # Insert text at current cursor position
             ConsoleBaseWidget.insert_text(self, text)
@@ -631,8 +642,14 @@ class ShellBaseWidget(ConsoleBaseWidget):
         event: the mouse press event (QMouseEvent)
         """
         if event.button() == Qt.MidButton:
-            self.setFocus()
-            self.paste()
+            text = self.selectedText()
+            # Simulating left mouse button:
+            event = QMouseEvent(QMouseEvent.MouseButtonPress, event.pos(),
+                                Qt.LeftButton, Qt.LeftButton, Qt.NoModifier)
+            ConsoleBaseWidget.mousePressEvent(self, event)
+            if self.new_input_line:
+                self.on_new_line()
+            self.insert_text(text)
             event.accept()
         else:
             ConsoleBaseWidget.mousePressEvent(self, event)
@@ -653,14 +670,20 @@ class ShellBaseWidget(ConsoleBaseWidget):
 
     def dropEvent(self, event):
         """Drag and Drop - Drop event"""
-        if(event.mimeData().hasFormat("text/plain")):
+        if (event.mimeData().hasFormat("text/plain")):
             text = event.mimeData().text()
+            if self.new_input_line:
+                self.on_new_line()
             self.insert_text(text, at_end=True)
             self.setFocus()
             event.setDropAction(Qt.MoveAction)
             event.accept()
         else:
             event.ignore()
+            
+    def drop_pathlist(self, pathlist):
+        """Drop path list"""
+        raise NotImplementedError
 
 
 class PythonShellWidget(ShellBaseWidget):
@@ -674,7 +697,7 @@ class PythonShellWidget(ShellBaseWidget):
     def __init__(self, parent, history_filename, debug=False, profile=False):
         ShellBaseWidget.__init__(self, parent, history_filename, debug, profile)
         
-        self.docviewer = None
+        self.inspector = None
         
         # Allow raw_input support:
         self.input_loop = None
@@ -714,12 +737,8 @@ class PythonShellWidget(ShellBaseWidget):
     def contextMenuEvent(self, event):
         """Reimplements ShellBaseWidget method"""
         state = self.hasSelectedText()
-        self.copy_action.setEnabled(state)
         self.copy_without_prompts_action.setEnabled(state)
-        self.cut_action.setEnabled(state)
-        self.menu.popup(event.globalPos())
-        event.accept()
-        
+        ShellBaseWidget.contextMenuEvent(self, event)
         
     def copy_without_prompts(self):
         """Copy text to clipboard without prompts"""
@@ -801,21 +820,18 @@ class PythonShellWidget(ShellBaseWidget):
         if self.is_completion_widget_visible():
             self.stdkey_tab()
         elif self.is_cursor_on_last_line():
-            buf = self.get_current_line_to_cursor()
-            empty_line = not buf.strip()
+            empty_line = not self.get_current_line_to_cursor().strip()
             if empty_line:
                 self.stdkey_tab()
-            elif buf.endswith('.'):
-                self.show_code_completion(self.get_last_obj())
-            elif buf[-1] in ['"', "'"]:
-                self.show_file_completion()
+            else:
+                self.show_code_completion()
                 
     def _key_home(self, shift):
         """Action for Home key"""
         if self.is_completion_widget_visible():
             self.completion_widget_home()
         elif self.is_cursor_on_last_line():
-            self.set_cursor_position(self.current_prompt_pos)
+            self.stdkey_home(shift, self.current_prompt_pos)
                 
     def _key_end(self, shift):
         """Action for End key"""
@@ -859,11 +875,12 @@ class PythonShellWidget(ShellBaseWidget):
         
     def _key_period(self, text):
         """Action for '.'"""
-        # Enable auto-completion only if last token isn't a float
-        last_obj = self.get_last_obj()
-        if last_obj and not last_obj.isdigit():
-            self.show_code_completion(last_obj)
         self.insert_text(text)
+        if self.codecompletion_auto:
+            # Enable auto-completion only if last token isn't a float
+            last_obj = self.get_last_obj()
+            if last_obj and not last_obj.isdigit():
+                self.show_code_completion()
 
 
     #------ Paste
@@ -884,22 +901,31 @@ class PythonShellWidget(ShellBaseWidget):
     
   
     #------ Code Completion / Calltips        
-    def show_completion_list(self, completions):
+    def show_completion_list(self, completions, completion_text=""):
         """Display the possible completions"""
-        if len(completions) == 0:
+        if len(completions) == 0 or completion_text in completions:
             return
-        self.show_completion_widget(sorted(completions))
-        self.completion_text = ""
-
-    def show_file_completion(self):
-        """Display a completion list for files and directories"""
-        self.show_completion_list(os.listdir(os.getcwdu()))
+        self.completion_text = completion_text
+        if isinstance(completions[0], unicode):
+            key = unicode.lower
+        else:
+            key = str.lower
+        self.show_completion_widget(sorted(completions, key=key))
 
     # Methods implemented in child class:
     # (e.g. InteractiveShell)
     def get_dir(self, objtxt):
         """Return dir(object)"""
         raise NotImplementedError
+    def get_completion(self, objtxt):
+        """Return completion list associated to object name"""
+        pass
+    def get_globals_keys(self):
+        """Return shell globals() keys"""
+        raise NotImplementedError
+    def get_cdlistdir(self):
+        """Return shell current directory list dir"""
+        raise NotImplementedError
     def iscallable(self, objtxt):
         """Is object callable?"""
         raise NotImplementedError
@@ -919,14 +945,64 @@ class PythonShellWidget(ShellBaseWidget):
         """Return True if object is defined"""
         raise NotImplementedError
         
-    def show_code_completion(self, text):
-        """Display a completion list based on the last token"""
-        if not self.codecompletion:
+    def show_code_completion(self):
+        """Display a completion list based on the current line"""
+        # Note: unicode conversion is needed only for ExternalShellBase
+        text = unicode(self.get_current_line_to_cursor())
+        last_obj = self.get_last_obj()
+        
+        #-- IPython only -------------------------------------------------------
+        # Using IPython code completion feature: __IP.complete
+        if ' ' in text and not text.endswith(' '):
+            try1 = text.split(' ')[-1]
+            obj_list = self.get_completion(try1)
+            if obj_list:
+                self.show_completion_list(obj_list, completion_text=try1)
+                return
+        obj_list = self.get_completion(last_obj)
+        if not text.endswith('.') and last_obj and obj_list:
+            self.show_completion_list(obj_list, completion_text=last_obj)
+            return
+        #-----------------------------------------------------------------------
+        
+        obj_dir = self.get_dir(last_obj)
+        if last_obj and obj_dir:
+            if not text.endswith('.'):
+                return
+            self.show_completion_list(obj_dir)
+            return
+        
+        # Builtins and globals
+        import re, __builtin__, keyword
+        if not text.endswith('.') and last_obj \
+           and re.match(r'[a-zA-Z_0-9]*$', last_obj):
+            b_k_g = dir(__builtin__)+self.get_globals_keys()+keyword.kwlist
+            if last_obj in b_k_g:
+                return
+            for objname in b_k_g:
+                if objname.startswith(last_obj):
+                    self.show_completion_list(b_k_g, completion_text=last_obj)
+        
+        # Looking for an incomplete completion
+        if last_obj is None:
+            last_obj = text
+        dot_pos = last_obj.rfind('.')
+        if dot_pos != -1:
+            if dot_pos == len(last_obj)-1:
+                completion_text = ""
+            else:
+                completion_text = last_obj[dot_pos+1:]
+                last_obj = last_obj[:dot_pos]
+            self.show_completion_list(self.get_dir(last_obj),
+                                      completion_text=completion_text)
+            return
+        
+        # Looking for ' or ": filename completion
+        q_pos = max([text.rfind("'"), text.rfind('"')])
+        if q_pos != -1:
+            self.show_completion_list(self.get_cdlistdir(),
+                                      completion_text=text[q_pos+1:])
             return
-        text = unicode(text) # Useful only for ExternalShellBase
-        objdir = self.get_dir(text)
-        if objdir:
-            self.show_completion_list(objdir) 
     
     def show_docstring(self, text, call=False):
         """Show docstring or arguments"""
@@ -935,11 +1011,11 @@ class PythonShellWidget(ShellBaseWidget):
         
         text = unicode(text) # Useful only for ExternalShellBase
         
-        if (self.docviewer is not None) and \
-           (self.docviewer.dockwidget.isVisible()):
-            # DocViewer widget exists and is visible
-            self.docviewer.refresh(text)
-            self.setFocus() # if docviewer was not at top level, raising it to
+        if (self.inspector is not None) and \
+           (self.inspector.dockwidget.isVisible()):
+            # ObjectInspector widget exists and is visible
+            self.inspector.set_object_text(text)
+            self.setFocus() # if inspector was not at top level, raising it to
                             # top will automatically give it focus because of
                             # the visibility_changed signal, so we must give
                             # focus back to shell
@@ -953,7 +1029,7 @@ class PythonShellWidget(ShellBaseWidget):
                             self.show_calltip(translate("PythonShellWidget",
                                                         "Arguments"),
                                               arglist, '#129625')
-        else: # docviewer is not visible
+        else: # inspector is not visible
             doc = self.get__doc__(text)
             if doc is not None:
                 self.show_calltip(translate("PythonShellWidget",
@@ -967,10 +1043,25 @@ class PythonShellWidget(ShellBaseWidget):
         """
         return getobj(self.get_current_line_to_cursor(), last=last)
         
-    def set_docviewer(self, docviewer):
-        """Set DocViewer DockWidget reference"""
-        self.docviewer = docviewer
-        self.docviewer.set_shell(self)
+    def set_inspector(self, inspector):
+        """Set ObjectInspector DockWidget reference"""
+        self.inspector = inspector
+        self.inspector.set_shell(self)
+            
+            
+    #------ Drag'n Drop
+    def drop_pathlist(self, pathlist):
+        """Drop path list"""
+        if pathlist:
+            files = ["r'%s'" % path for path in pathlist]
+            if len(files) == 1:
+                text = files[0]
+            else:
+                text = "[" + ", ".join(files) + "]"
+            if self.new_input_line:
+                self.on_new_line()
+            self.insert_text(text)
+            self.setFocus()
 
 
 class TerminalWidget(ShellBaseWidget):
@@ -1009,7 +1100,7 @@ class TerminalWidget(ShellBaseWidget):
     def _key_home(self, shift):
         """Action for Home key"""
         if self.is_cursor_on_last_line():
-            self.set_cursor_position(self.current_prompt_pos)
+            self.stdkey_home(shift, self.current_prompt_pos)
                 
     def _key_end(self, shift):
         """Action for End key"""
@@ -1039,3 +1130,18 @@ class TerminalWidget(ShellBaseWidget):
     def _key_period(self, text):
         """Action for '.'"""
         self.insert_text(text)
+            
+            
+    #------ Drag'n Drop
+    def drop_pathlist(self, pathlist):
+        """Drop path list"""
+        if pathlist:
+            files = ['"%s"' % path for path in pathlist]
+            if len(files) == 1:
+                text = files[0]
+            else:
+                text = " ".join(files)
+            if self.new_input_line:
+                self.on_new_line()
+            self.insert_text(text)
+            self.setFocus()
